# 7.2.2 Konstruktion des Erhebungsdesigns

# Generierung eines Versuchsplans mit R (Cola_Versuchsplan.r)
# Vollstndiger Versuchsplan mit Berechnung der D-Effizienz:
all=expand.grid(A=c("1","2","3"),B=c("1","2","3","4"),
                C=c("1","2","3"),D=c("1","2"),
                E=c("1","2","3","4"),F=c("1","2","3","4"))
set.seed(1); n=dim(all)[1]; library(AlgDesign)
plan.all=optFederov(~.,data=all,nTrials=n); plan.all$D
# Reduzierter Versuchsplan nach Federov:
plan.red=optFederov(~.,data=all,nTrials=25,nRepeats=100)
plan.red$design; plan.red$D; plan.red$D/plan.all$D
# (zustzliche) "von Hand"-Berechnung der D-Effizienz:
X=as.matrix(model.matrix(~.,all)); n=dim(X)[1]
XsX=t(X)%*%X; XsXinv=solve(XsX); m=dim(X)[2]
plan.all.D=1/(n*det(XsXinv)**(1/m)); plan.all.D
X=as.matrix(model.matrix(~.,plan.red$design)); n=dim(X)[1]
XsX=t(X)%*%X; XsXinv=solve(XsX); m=dim(X)[2]
plan.red.D=1/(n*det(XsXinv)**(1/m)); plan.red.D
plan.red.D/plan.all.D

# 7.2.4 Berechnung der Nutzenbeitrge und Wichtigkeiten

# Traditionelle Conjointanalyse mit R (Datei cola_TCA.r)
setwd("C:/Users/Alpha/Conjointanalyse/Cola/R")
# 1. Einlesen der Sortierung der Stimuli als Sequenz von Nummern
I=199; J=25; K=6; L=c(3,4,3,2,4,4); p=1+sum(L)
Plan=read.csv("cola_plan.csv",sep=";"); Plan=Plan[1:J,1:K]
Data=read.csv("cola_data.csv",sep=";"); Data=Data[,24:52]
for (k in 1:K) {Plan[,k]=as.factor(Plan[,k])}
# 2. Berechnung der Nutzenbeitrge und der Wichtigkeiten:
y=matrix(0,nrow=I,ncol=J); u=matrix(0,nrow=I,ncol=p)
w=matrix(0,nrow=I,ncol=K); for (i in 1:I){# ber alle Personen
  # a. Umwandlung der Sequenz in Prferenzwerte
  pwert=J; for (ix in 1:29){nummer=Data[i,ix];
  if (nummer<=J) {y[i,nummer]=pwert; pwert=pwert-1}}
  # b. Kleinste-Quadrate-Schaetzung mit lm (linear model)
  Plan$pwert=y[i,]; fit=lm(pwert~.,data=Plan)
  # c. Uebertragung normierter Ergebnisse nach u und w:
  b=fit$coefficients; ix=1; u[i,ix]=b[ix]; wsum=0
  for (k in 1:K) {for (l in 2:L[k]) {u[i,ix+l]=b[ix-k+l]}
    w[i,k]=max(u[i,(ix+1):(ix+L[k])])-min(u[i,(ix+1):(ix+L[k])])
    wsum=wsum+w[i,k]; delta=sum(u[i,(ix+1):(ix+L[k])])/L[k]
    for (l in 1:L[k]) {u[i,ix+l]=u[i,ix+l]-delta}
    ; u[i,1]=u[i,1]+delta; ix=ix+L[k]}
    for (k in 1:K) {w[i,k]=w[i,k]/wsum}
}; for (k in 1:K) {print(mean(w[,k]))} # mittlere Wichtigkeiten
for (l in 1:p) {print(mean(u[,l]))} # mittlere Nutzenbeitrge
# 3. Ausgabe der Prferenzwerte in eine csv-Datei:
id=1:199; id1=id; for (i in 1:199){id1[i]=1}; y=cbind(id,id1,y)
y=as.data.frame(y); write.csv(y,file="cola_pwert.csv",row.names=FALSE)
# 4. Validittsprfung (First-Choice-Hitrate):
I=199; J=32; K=6; L=c(3,4,3,2,4,4); T=8; Js=4
Data=read.csv("cola_data.csv",sep=";"); Data=Data[,53:60]
Plan=read.csv("cola_plan.csv",sep=";"); Plan=Plan[1:32,]
ys=matrix(0,nrow=I,ncol=J); hc=matrix(0,nrow=T,ncol=Js)
for (j in 1:J){hc[Plan[j,9],Plan[j,10]]=j}; n1=0; n2=0; ysh=1:Js
for (i in 1:I){# ber alle Auskunftspersonen:
  for (j in 1:J){# Berechnen der Prferenzwerte der J Stimuli:
    ix=1; ys[i,j]=u[i,ix]
    for (k in 1:K){
      ys[i,j]=ys[i,j]+u[i,ix+Plan[j,k]]; ix=ix+L[k]}}
  for (t in 1:T){# berprfung der T Holdout-Choice-Sets:
    for (j in 1:Js){ysh[j]=ys[i,hc[t,j]]}; j_it=Data[i,t]
    if (j_it<5){n2=n2+1; if (ysh[j_it]==max(ysh)){n1=n1+1}}}}
print(c("First-Choice-Hitrate:", n1/n2)) # richtig/gesamt

# Traditionelle Conjointanalyse mit R: HB-Schtzung (cola_TCA_HB.r)
# 1. Einlesen der Prferenzwerte und der Stimulusbeschreibungen:
setwd("C:/Users/Alpha/Conjointanalyse/Cola/R")
I=199; J=25; K=6; L=c(3,4,3,2,4,4); p=1+sum(L)
Data=read.csv("cola_pwert.csv",sep=","); Data=as.matrix(Data[,3:27])
Plan=read.csv("cola_plan.csv",sep=";"); Plan=Plan[1:J,1:K]
# 2. HB-Schtzung der Teilnutzenwerte (Gibbs Sampling)
library(bayesm); regdata=NULL
for (k in 1:K){Plan[,k]=as.factor(Plan[,k])}
X=as.matrix(model.matrix(~.,data=Plan))
for (i in 1:I) {y=as.vector(t(Data[i,]));regdata[[i]]=list(y=y, X=X)}
Data=list(regdata=regdata); Mcmc <- list(R=20000)
fit=rhierLinearModel(Data=Data,Mcmc=Mcmc)
# 3. bertragung normierter Werte nach u und w:
u=matrix(0,nrow=I,ncol=p); w=matrix(0,nrow=I,ncol=K); b=1:(1+sum(L)-K)
for (i in 1:I){# ber alle Personen
  for (j in 1:1+sum(L)-K){b[j]=mean(as.vector(fit$betadraw[i,j,
           10001:20000]))}
  ix=1; u[i,ix]=b[ix]; wsum=0
  for (k in 1:K) {for (l in 2:L[k]) {u[i,ix+l]=b[ix-k+l]}
    w[i,k]=max(u[i,(ix+1):(ix+L[k])])-min(u[i,(ix+1):(ix+L[k])])
    wsum=wsum+w[i,k]; delta=sum(u[i,(ix+1):(ix+L[k])])/L[k]
    for (l in 1:L[k]) {u[i,ix+l]=u[i,ix+l]-delta}
    u[i,1]=u[i,1]+delta; ix=ix+L[k]}
  for (k in 1:K) {w[i,k]=w[i,k]/wsum}
}; for (k in 1:K) {print(mean(w[,k]))} # mittlere Wichtigkeiten
for (l in 1:p) {print(mean(u[,l]))} # mittlere Nutzenbeitrge
